
class LayerInterface:
    
    @classmethod
    def auto(cls, **kwargs) -> "LayerInterface":
        raise NotImplementedError("Auto instantiation not implemented for base LayerInterface")

    def run(self, **kwargs) -> bool:
        raise NotImplementedError("Run method not implemented for base LayerInterface")



import enum

from fred.ogd.layer.interface import LayerInterface
from fred.ogd.layer._landing import LayerLanding


class LayerCatalog(enum.Enum):
    LANDING = LayerLanding

    def auto(self, **kwargs) -> LayerInterface:
        match self:
            case self.LANDING:
                return LayerLanding.auto(**kwargs)
            case _:
                raise NotImplementedError(f"Auto instantiation not implemented for {self.name}")



import os
from typing import Optional
from dataclasses import dataclass

from fred.dao.comp.catalog import CompCatalog, FredKeyVal
from fred.ogd.source.catalog import SourceCatalog
from fred.ogd.source.interface import SourceInterface
from fred.ogd.layer.interface import LayerInterface
from fred.ogd.source.model import Snapshot


@dataclass(frozen=True, slots=True)
class LayerLanding(LayerInterface):
    source: SourceInterface
    keyval: FredKeyVal

    @classmethod
    def auto(
            cls,
            source: str,
            backend: str,
            source_kwargs: Optional[dict] = None,
            backend_kwargs: Optional[dict] = None,
    ) -> "LayerLanding":
        source_instance = SourceCatalog[source].auto(**(source_kwargs or {}))
        keyval_instance = CompCatalog.KEYVAL.mount(srv_ref=backend, **(backend_kwargs or {}))
        return cls(source=source_instance, keyval=keyval_instance)
    
    def snapshot(self, **kwargs) -> Snapshot:
        return self.source.snapshot(**kwargs)

    def run(
            self,
            output_path: Optional[str] = None,
            ts_format: Optional[str] = None,
            include_uuid: bool = False,
            snapshot_kwargs: Optional[dict] = None,
            key_setting_kwargs: Optional[dict] = None,
    ) -> bool:
        snapshot = self.snapshot(**(snapshot_kwargs or {}))
        filename = snapshot.filename(ts_format=ts_format, include_uuid=include_uuid)
        content = snapshot.content()
        snapshot_key = self.keyval(key=os.path.join(output_path or "", filename))
        if not snapshot_key.get():
            snapshot_key.set(content, **(key_setting_kwargs or {}))
            return True
        return False



from dataclasses import dataclass

import requests

from fred.settings import logger_manager
from fred.ogd.source.interface import SourceInterface

logger = logger_manager.get_logger(__name__)


@dataclass(frozen=True, slots=True)
class SourceRequest(SourceInterface):
    target_url: str

    def request(self, **kwargs) -> requests.Response:
        response = requests.get(self.target_url, **kwargs)
        response.raise_for_status()
        if not response.ok:
            raise requests.HTTPError(f"Request failed with status code {response.status_code}")
        return response

    def fetch_snapshot_data(self, **kwargs) -> dict | str:
        response = self.request(**kwargs)
        try:
            return response.json()
        except requests.JSONDecodeError:
            return response.text
        except Exception as e:
            logger.error(f"An error occurred during fetch: {e}")
            raise e



from fred.ogd.source.model import Snapshot


class SourceInterface:
    
    def snapshot(self, **kwargs) -> Snapshot:
        snapshot_data = self.fetch_snapshot_data(**kwargs)
        return Snapshot.auto(
            data=snapshot_data,
            **kwargs
        )



import os
import uuid
import json
import datetime as dt
from typing import Optional

from pydantic import BaseModel


class Snapshot(BaseModel):
    data: dict | str
    created_at: str
    metadata: dict

    @classmethod
    def auto(cls, data: dict | str, created_at: Optional[str] = None, metadata: Optional[dict] = None) -> "Snapshot":
        from fred.utils.dateops import datetime_utcnow
        return cls(
            data=data,
            created_at=created_at or datetime_utcnow().isoformat(),
            metadata=metadata or {},
        )

    @property
    def dt_created_at(self) -> dt.datetime:
        return dt.datetime.fromisoformat(self.created_at)

    @property
    def data_uuid(self) -> str:
        return str(uuid.uuid5(uuid.NAMESPACE_OID, self.data if isinstance(self.data, str) else json.dumps(self.data, default=str)))

    def filename(self, ts_format: Optional[str] = None, include_uuid: bool = False) -> str:
        ts_format = ts_format or "%Y-%m-%d-%H-%M-%S"
        data_id = f"-{self.data_uuid}" if include_uuid else ""
        return f"snapshot-{self.dt_created_at.strftime(ts_format)}{data_id}.json"

    def content(self) -> str:
        return self.model_dump_json()

    def save(self, path: str, filename: Optional[str] = None, ts_format: Optional[str] = None, include_uuid: bool = False) -> None:
        filename = filename or self.filename(ts_format=ts_format, include_uuid=include_uuid)
        filepath = os.path.join(path, filename)
        with open(filepath, "w") as file:
            file.write(self.content())



import enum

from fred.ogd.source.interface import SourceInterface
from fred.ogd.source._request import SourceRequest


class SourceCatalog(enum.Enum):
    REQUEST = SourceRequest

    def auto(self, **kwargs) -> SourceInterface:
        match self:
            case self.REQUEST:
                return SourceRequest(**kwargs)
            case _:
                raise NotImplementedError(f"Auto instantiation not implemented for {self.name}")



import os
import json
from setuptools import setup, find_namespace_packages
from dataclasses import dataclass


OGD_TARGET_PROJECT = os.environ.get(
    "OGD_TARGET_PROJECT",
    default="main",
)


@dataclass(frozen=True, slots=True)
class OGDProject:
    name: str
    codebase_path: str
    configs_path: str
    spec_path: str
    readme_path: str

    @classmethod
    def project_names(cls) -> list[str]:
        return [
            proj for proj in os.listdir("configs")
            if os.path.isdir(os.path.join("configs", proj))
        ]

    @classmethod
    def projects(cls) -> dict[str, "OGDProject"]:
        return {
            proj: cls(
                name=proj,
                codebase_path=os.path.join("src", proj),
                configs_path=os.path.join("configs", proj),
                spec_path=os.path.join("configs", proj, "spec.json"),
                readme_path=os.path.join("configs", proj, "README.md"),
            )
            for proj in cls.project_names()
        }

    @property
    def readme(self) -> str:
        with open(self.readme_path, "r") as file:
            return file.read()
        
    @property
    def spec(self) -> dict:
        with open(self.spec_path, "r") as file:
            return json.load(file)
    
    @property
    def package_name(self) -> str:
        return self.spec.get("name", f"fred-ogd-{self.name}")

    @property
    def version_filepath(self) -> str:
        return os.path.join(
            self.codebase_path,
            "fred",
            "ogd", 
            "" if self.name == "main" else self.name,
            "version"
        )

    @property
    def version(self) -> str:
        with open(self.version_filepath, "r") as file:
            return file.read().strip()

    @property
    def major_version(self) -> int:
        major, *_ = self.version.split(".")
        return int(major)
    @property
    def requirements(self) -> list[str]:
        return self.spec.get("requirements", [])


ogd_projects = OGDProject.projects()

if OGD_TARGET_PROJECT not in ogd_projects.keys():
    raise ValueError(
        f"OGD_PROJECT '{OGD_TARGET_PROJECT}' is not valid. Available projects: {ogd_projects.keys()}"
    )

ogd_project = ogd_projects[OGD_TARGET_PROJECT]


setup(
    name=ogd_project.package_name,
    version=ogd_project.version,
    description=f"FRED-OGD: {ogd_project.name.upper()}",
    long_description=ogd_project.readme,
    long_description_content_type='text/markdown',
    url="https://ogd.fred.fahera.mx",
    author="Fahera Research, Education, and Development",
    author_email="fred@fahera.mx",
    packages=find_namespace_packages(where=ogd_project.codebase_path),
    package_dir={
        "": ogd_project.codebase_path
    },
    package_data={
        "": [
            ogd_project.version_filepath,
            "**/*.json",
            "**/*.yaml",
        ]
    },
    install_requires=ogd_project.requirements,
    extras_require={
        "standalone": [
            f"fred-ogd<={ogd_projects['main'].major_version + 1}.0.0" 
        ]
    } if OGD_TARGET_PROJECT != "main" else {
        key: [f"{proj.package_name}<={proj.major_version + 1}.0.0"]
        for key, proj in ogd_projects.items()
        if key != "main"
    },
    include_package_data=True,
    entry_points={
        "console_scripts": ogd_project.spec.get("console_scripts", []),
    },
    # Python version should be aligned to the latest databricks LTS runtime at the moment.
    # For more info: https://docs.databricks.com/aws/en/release-notes/runtime/ 
    # Reference: Databricks Runtime 16.4 LTS Using Python 3.12.3
    python_requires=">=3.12",
)



from fred.cli.interface import AbstractCLI


class OGDCLI(AbstractCLI):

    def version(self):
        from fred.ogd.version import version
        return version

    def execute(self, name: str):
        import importlib

        module = importlib.import_module(f"fred.ogd.{name}.cli")
        with module.OGDExtCLI.default_config() as cli:
            return cli
